"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _lodash = _interopRequireDefault(require("lodash"));

var _http = _interopRequireDefault(require("http"));

var _https = _interopRequireDefault(require("https"));

require("raf/polyfill");

var _server = _interopRequireDefault(require("./server"));

var _renderer = require("./renderer");

/**
 * Standard web servers.
 */

/* Polyfill required by ReactJS. */

/**
 * Normalizes a port into a number, string, or false.
 * @param {String} value Port name or number.
 * @return Port number (Number), name (String), or false.
 */
function normalizePort(value) {
  var port = _lodash.default.toNumber(value);

  if (_lodash.default.isFinite(port)) return port;
  /* port number */

  if (!_lodash.default.isNumber(port)) return value;
  /* named pipe */

  return false;
}
/**
 * Creates and starts a new webserver.
 * @param {Object} webpackConfig The Webpack config that was used to build the
 *  frontend bundle.
 * @param {config} options Additional options:
 *  - Application {Function} - Optional. The root ReactJS component of the app
 *    to use for server-side rendering;
 *  - devMode {Boolean} - Whether the server should be started in dev mode;
 *  - favicon {String} - Path of the favicon to be used by the server;
 *  - logger {Object} - Optional. The logger to use. By default, the console
 *    is used (which is not a good decision performancewise, but it will be
 *    changed soon);
 *  - https {Object} - Optional. If given, HTTPS server will be started with
 *    the specified settings. HTTP server that hanles HTTP > HTTPS redirection
 *    will be also started at the specified port. Expected fields:
 *    - cert {String} - SSL Certificate;
 *    - key {String} - SSL key;
 *  - beforeRender {Function} - The hook into server-side rendering. It will get
 *    incoming request as the argument and it should return a promise that will
 *    resolve to the object with the following fields all optional:
 *    - config {Object} - Config object to be injected into the page;
 *    - extraScripts {Array} - Any additional scripts to be injected into
 *      HTML template;
 *    - store {Object} - Redux store, which state should be used for server-side
 *      rendering, if it is performed, and also injected into HTML template as
 *      the initial state.
 *  - onExpressJsSetup {Function} - Custom setup of ExpressJS server.
 *  - port {String} - Optional. The port to listen (number or name). If not
 *    specified the value will be taken from PORT environmental variable, or
 *    default to 3000 otherwise.
 * @return {Promise} Resolves to the result object has two fields:
 *  - express {Object} - ExpressJS server;
 *  - httpServer {Object} - NodeJS HTTP(S) server.
 */


function launch(_x, _x2) {
  return _launch.apply(this, arguments);
}

function _launch() {
  _launch = (0, _asyncToGenerator2.default)(
  /*#__PURE__*/
  _regenerator.default.mark(function _callee(webpackConfig, options) {
    var ops, expressServer, httpServer;
    return _regenerator.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            /* Options normalization. */
            ops = options ? _lodash.default.clone(options) : {};
            ops.port = normalizePort(ops.port || process.env.PORT || 3000);

            _lodash.default.defaults(ops, {
              logger: console
            });
            /* Creates servers, resolves and sets the port. */


            _context.next = 5;
            return (0, _server.default)(webpackConfig, ops);

          case 5:
            expressServer = _context.sent;

            if (ops.https) {
              httpServer = _https.default.createServer({
                cert: ops.https.cert,
                key: ops.https.key
              }, expressServer);
            } else httpServer = _http.default.createServer(expressServer);
            /* Sets error handler for HTTP(S) server. */


            httpServer.on('error', function (error) {
              if (error.syscall !== 'listen') throw error;
              var bind = _lodash.default.isString(ops.port) ? "Pipe ".concat(ops.port) : "Port ".concat(ops.port);
              /* Human-readable message for some specific listen errors. */

              switch (error.code) {
                case 'EACCES':
                  ops.logger.error("".concat(bind, " requires elevated privileges"));
                  process.exit(1);
                  break;

                case 'EADDRINUSE':
                  ops.logger.error("".concat(bind, " is already in use"));
                  process.exit(1);
                  break;

                default:
                  throw error;
              }
            });
            /* Listening event handler for HTTP(S) server. */

            httpServer.on('listening', function () {
              var addr = httpServer.address();
              var bind = _lodash.default.isString(addr) ? "pipe ".concat(addr) : "port ".concat(addr.port);
              ops.logger.info("Server listening on ".concat(bind, " in ").concat(process.env.NODE_ENV, " mode"));
            });
            httpServer.listen(ops.port);
            return _context.abrupt("return", {
              expressServer: expressServer,
              httpServer: httpServer
            });

          case 11:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }));
  return _launch.apply(this, arguments);
}

launch.SCRIPT_LOCATIONS = _renderer.SCRIPT_LOCATIONS;
var _default = launch;
exports.default = _default;