"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = factory;
exports.SCRIPT_LOCATIONS = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _lodash = _interopRequireDefault(require("lodash"));

var _config = _interopRequireDefault(require("config"));

var _nodeForge = _interopRequireDefault(require("node-forge"));

var _fs = _interopRequireDefault(require("fs"));

var _moment = _interopRequireDefault(require("moment"));

var _path = _interopRequireDefault(require("path"));

var _react = _interopRequireDefault(require("react"));

var _server = _interopRequireDefault(require("react-dom/server"));

var _serializeJavascript = _interopRequireDefault(require("serialize-javascript"));

var _reactHelmet = require("react-helmet");

var _reactRedux = require("react-redux");

var _reactRouterDom = require("react-router-dom");

/**
 * ExpressJS middleware for server-side rendering of a ReactJS app.
 */
var sanitizedConfig = _lodash.default.omit(_config.default, 'SECRET');

var SCRIPT_LOCATIONS = {
  BODY_OPEN: 'BODY_OPEN',
  DEFAULT: 'DEFAULT',
  HEAD_OPEN: 'HEAD_OPEN'
};
/**
 * Reads build-time information about the app. This information is generated
 * by our standard Webpack config for apps, and it is written into
 * ".build-info" file in the context folder specified in Webpack config.
 * At the moment, that file contains build timestamp and a random 32-bit key,
 * suitable for cryptographical use.
 * @param {String} context Webpack context path used during the build.
 * @return {Object} Resolves to the build-time information.
 */

exports.SCRIPT_LOCATIONS = SCRIPT_LOCATIONS;

function getBuildInfo(context) {
  var url = _path.default.resolve(context, '.build-info');

  return JSON.parse(_fs.default.readFileSync(url));
}
/**
 * Attempts to read from the disk Webpack stats generated during the build.
 * It will not work for development builds, where these stats should be captured
 * via compilator callback.
 * @param {String} buildDir
 * @return {Object} Resolves to the stats, or null, if cannot be read.
 */


function getWebpackStats(buildDir) {
  var url = _path.default.resolve(buildDir, '__stats__.json');

  var res;

  try {
    res = JSON.parse(_fs.default.readFileSync(url));
  } catch (err) {
    res = null;
  }

  return res;
}
/**
 * Prepares a new Cipher for data encryption.
 * @param {String} key Encryption key (32-bit random key is expected, see
 *  node-forge documentation, in case of doubts).
 * @return {Promise} Resolves to the object with two fields:
 *  1. cipher - a new Cipher, ready for encryption;
 *  2. iv - initial vector used by the cipher.
 */


function prepareCipher(key) {
  return new Promise(function (resolve, reject) {
    _nodeForge.default.random.getBytes(32, function (err, iv) {
      if (err) reject(err);else {
        // console.log('KEY', key);
        var cipher = _nodeForge.default.cipher.createCipher('AES-CBC', key);

        cipher.start({
          iv: iv
        });
        resolve({
          cipher: cipher,
          iv: iv
        });
      }
    });
  });
}
/**
 * Creates the middleware.
 * @param {Object} webpackConfig
 * @param {Object} options Additional options:
 * @return {Function} Created middleware.
 */


function factory(webpackConfig, options) {
  var buildInfo = getBuildInfo(webpackConfig.context);
  global.TRU_BUILD_INFO = buildInfo;
  var WEBPACK_STATS = getWebpackStats(webpackConfig.output.path);

  var ops = _lodash.default.defaults(_lodash.default.clone(options), {
    beforeRender: function beforeRender() {
      return Promise.resolve({});
    }
  });

  return (
    /*#__PURE__*/
    function () {
      var _ref = (0, _asyncToGenerator2.default)(
      /*#__PURE__*/
      _regenerator.default.mark(function _callee(req, res, next) {
        var _ref2, _ref3, _ref3$, configToInject, extraScripts, store, _ref3$2, cipher, iv, context, helmet, App, INJ, publicPath, assetsByChunkName, webpackStats, _webpackStats$toJson, timestamp, styles, bodyOpenExtraScripts, defaultExtraScripts, headOpenExtraScripts;

        return _regenerator.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.prev = 0;
                _context.next = 3;
                return Promise.all([ops.beforeRender(req, sanitizedConfig), prepareCipher(buildInfo.key)]);

              case 3:
                _ref2 = _context.sent;
                _ref3 = (0, _slicedToArray2.default)(_ref2, 2);
                _ref3$ = _ref3[0];
                configToInject = _ref3$.configToInject;
                extraScripts = _ref3$.extraScripts;
                store = _ref3$.store;
                _ref3$2 = _ref3[1];
                cipher = _ref3$2.cipher;
                iv = _ref3$2.iv;

                /* Context for react-router and collection of data related to server-side
                * rendering (this will be moved into separate place in future). */
                context = {
                  /* Array of chunk names, to use for stylesheet links injection. */
                  chunks: [],

                  /* Pre-rendered HTML markup for dynamic chunks. */
                  splits: {},
                  store: store
                };

                /* Optional server-side rendering. */
                App = options.Application;

                if (App) {
                  App = _react.default.createElement(_reactRouterDom.StaticRouter, {
                    context: context,
                    location: req.url
                  }, _react.default.createElement(App, null));

                  if (store) {
                    App = _react.default.createElement(_reactRedux.Provider, {
                      store: store
                    }, App);
                  }

                  App = _server.default.renderToString(App);
                  /* This takes care about server-side rendering of page title and meta tags
                  * (still demands injection into HTML template, which happens below). */

                  helmet = _reactHelmet.Helmet.renderStatic();
                }
                /* Encrypts data to be injected into HTML.
                * Keep in mind, that this encryption is no way secure: as the JS bundle
                * contains decryption key and is able to decode it at the client side.
                * Hovewer, for a number of reasons, encryption of injected data is still
                * better than injection of a plain text. */


                cipher.update(_nodeForge.default.util.createBuffer(JSON.stringify({
                  CONFIG: configToInject || sanitizedConfig,
                  ISTATE: store ? store.getState() : null
                }), 'utf8'));
                cipher.finish();
                INJ = _nodeForge.default.util.encode64("".concat(iv).concat(cipher.output.data));
                /* It is supposed to end with '/' symbol as path separator. */

                publicPath = webpackConfig.output.publicPath;
                webpackStats = res.locals.webpackStats;

                if (webpackStats) {
                  _webpackStats$toJson = webpackStats.toJson();
                  assetsByChunkName = _webpackStats$toJson.assetsByChunkName;
                } else if (WEBPACK_STATS) {
                  assetsByChunkName = WEBPACK_STATS.assetsByChunkName;
                } else assetsByChunkName = {};

                timestamp = (0, _moment.default)(buildInfo.timestamp).valueOf();
                if (context.status) res.status(context.status);
                styles = [];
                context.chunks.forEach(function (chunk) {
                  var assets = assetsByChunkName[chunk];
                  if (!assets) return;
                  if (!_lodash.default.isArray(assets)) assets = [assets];
                  assets = assets.filter(function (asset) {
                    return asset.endsWith('.css');
                  });
                  assets.forEach(function (asset) {
                    styles.push("<link data-chunk=\"".concat(chunk, "\" id=\"tru-style\" href=\"").concat(publicPath).concat(asset, "\" rel=\"stylesheet\" />"));
                  });
                });

                if (extraScripts) {
                  bodyOpenExtraScripts = extraScripts.filter(function (script) {
                    return _lodash.default.isObject(script) && script.location === SCRIPT_LOCATIONS.BODY_OPEN;
                  }).map(function (script) {
                    return script.code;
                  }).join('');
                  defaultExtraScripts = extraScripts.filter(function (script) {
                    return _lodash.default.isString(script) || script.location === SCRIPT_LOCATIONS.DEFAULT;
                  }).map(function (script) {
                    return _lodash.default.isString(script) ? script : script.code;
                  }).join('');
                  headOpenExtraScripts = extraScripts.filter(function (script) {
                    return _lodash.default.isObject(script) && script.location === SCRIPT_LOCATIONS.HEAD_OPEN;
                  }).map(function (script) {
                    return script.code;
                  }).join('');
                }

                res.send("<!DOCTYPE html>\n        <html lang=\"en\">\n          <head>\n            ".concat(headOpenExtraScripts || '', "\n            ").concat(helmet ? helmet.title.toString() : '', "\n            ").concat(helmet ? helmet.meta.toString() : '', "\n            <meta name=\"theme-color\" content=\"#FFFFFF\"/>\n            <link rel=\"manifest\" href=\"").concat(publicPath, "manifest.json\">\n            <link\n              href=\"").concat(publicPath, "main-").concat(timestamp, ".css\"\n              id=\"tru-style\"\n              rel=\"stylesheet\"\n            />\n            ").concat(styles.join(''), "\n            <link rel=\"shortcut icon\" href=\"/favicon.ico\" />\n            <meta charset=\"utf-8\" />\n            <meta\n              content=\"width=device-width,initial-scale=1.0\"\n              name=\"viewport\"\n            />\n          </head>\n          <body>\n            ").concat(bodyOpenExtraScripts || '', "\n            <div id=\"react-view\">").concat(App || '', "</div>\n            <script id=\"inj\" type=\"application/javascript\">\n              window.SPLITS = ").concat((0, _serializeJavascript.default)(context.splits, {
                  isJSON: true
                }), "\n              window.INJ=\"").concat(INJ, "\"\n            </script>\n            <script\n              src=\"").concat(publicPath, "polyfills-").concat(timestamp, ".js\"\n              type=\"application/javascript\"\n            ></script>\n            ").concat(defaultExtraScripts || '', "\n            <script\n              src=\"").concat(publicPath, "main-").concat(timestamp, ".js\"\n              type=\"application/javascript\"\n            ></script>\n          </body>\n        </html>"));
                _context.next = 32;
                break;

              case 29:
                _context.prev = 29;
                _context.t0 = _context["catch"](0);
                next(_context.t0);

              case 32:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this, [[0, 29]]);
      }));

      return function (_x, _x2, _x3) {
        return _ref.apply(this, arguments);
      };
    }()
  );
}